import re
from collections.abc import Callable, Collection, Iterable, Iterator, Mapping, Sequence
from functools import cached_property
from typing import Literal, TypedDict, TypeVar, type_check_only
from typing_extensions import Required
from typing import TypeAlias
from urllib.request import Request
from xml.etree import ElementTree as ET

from ..cache import Cache
from ..cookies import LenientSimpleCookie, YoutubeDLCookieJar
from ..networking.common import Response, _RequestData
from ..networking.impersonate import ImpersonateTarget
from ..utils._utils import NO_DEFAULT, RetryManager as _RetryManager
from ..YoutubeDL import YoutubeDL


@type_check_only
class _InfoDict(TypedDict, total=False):
    age_limit: int
    availability: Literal['private', 'premium_only', 'subscriber_only', 'needs_auth', 'unlisted',
                          'public'] | None
    creator: str | None
    comment_count: int | None
    duration: int | None
    formats: list[object] | None
    id: Required[str]
    like_count: int | None
    tags: list[str] | None
    thumbnail: str | None
    timestamp: int | float | None
    title: str | None
    uploader: str | None
    url: str | None


_StrNoDefaultOrNone: TypeAlias = str | type[NO_DEFAULT] | None
_T = TypeVar('_T')


class InfoExtractor:
    IE_DESC: str | bool
    SEARCH_KEY: str

    def _login_hint(self, method: _StrNoDefaultOrNone, netrc: str | None = None) -> dict[str, str]:
        ...

    def __init__(self, downloader: YoutubeDL | None = None) -> None:
        ...

    @classmethod
    def _match_valid_url(cls, url: str) -> re.Match[str] | None:
        ...

    @classmethod
    def suitable(cls, url: str) -> re.Match[str] | None:
        ...

    @classmethod
    def get_temp_id(cls, url: str) -> str | None:
        ...

    @classmethod
    def working(cls) -> bool:
        ...

    @classmethod
    def supports_login(cls) -> bool:
        ...

    def initialize(self) -> None:
        ...

    def extract(self, url: str) -> Iterator[_InfoDict]:
        ...

    def set_downloader(self, downloader: YoutubeDL) -> None:
        ...

    @property
    def cache(self) -> Cache:
        ...

    @property
    def cookiejar(self) -> YoutubeDLCookieJar:
        ...

    def _initialize_pre_login(self) -> None:
        ...

    def _perform_login(self, username: str, password: str) -> None:
        ...

    def _real_initialize(self) -> None:
        ...

    @classmethod
    def ie_key(cls) -> str:
        ...

    @property
    def IE_NAME(cls) -> str:
        ...

    def _create_request(
        self,
        url_or_request: str | Request,
        data: object | None = None,
        headers: Mapping[str, str] | None = None,
        query: str | Mapping[str, str] | None = None,
        extensions: Mapping[str, object] | None = None,
    ) -> Request:
        ...

    def _download_webpage_handle(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        encoding: str | None = None,
        data: object | None = None,
        headers: Mapping[str, str] | None = None,
        query: str | Mapping[str, str] | None = None,
        expected_status: int | None = None,
        impersonate: ImpersonateTarget | str | bool | Collection[str | ImpersonateTarget]
        | None = None,
        require_impersonation: bool = False,
    ) -> tuple[str, Response] | Literal[False]:
        ...

    @staticmethod
    def _guess_encoding_from_content(content_type: str, webpage_bytes: bytes) -> str:
        ...

    def _webpage_read_content(
        self,
        urlh: Response,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        prefix: str | None = None,
        encoding: str | None = None,
        data: _RequestData | None = None,
    ) -> str | Literal[False]:
        ...

    def _parse_json(
        self,
        json_string: str,
        video_id: str,
        transform_source: Callable[..., str] | None = None,
        fatal: bool = True,
        errnote: str | None = None,
        **parser_kwargs: object,
    ) -> object:
        ...

    def _parse_socket_response_as_json(self, data: str, *args: object, **kwargs: object) -> object:
        ...

    def report_warning(self,
                       msg: str,
                       video_id: str | None = None,
                       *args: object,
                       only_once: bool = False,
                       **kwargs: object) -> None:
        ...

    def to_screen(self, msg: str, *args: object, **kwargs: object) -> None:
        ...

    def write_debug(self, msg: str, *args: object, **kwargs: object) -> None:
        ...

    def get_param(self,
                  name: str,
                  default: object | None = None,
                  *args: object,
                  **kwargs: object) -> object:
        ...

    def report_drm(self, video_id: str) -> None:
        ...

    def report_extraction(self, id_or_name: str) -> None:
        ...

    def report_download_webpage(self, video_id: str) -> None:
        ...

    def report_age_confirmation(self) -> None:
        ...

    def report_login(self) -> None:
        ...

    def raise_login_required(
        self,
        msg: str = 'This video is only available for registered users',
        metadata_available: bool = False,
        method: str | type[NO_DEFAULT] = ...,
    ) -> None:
        ...

    def raise_geo_restricted(self,
                             msg: str = ...,
                             countries: Collection[str] | None = None,
                             metadata_available: bool = False) -> None:
        ...

    def raise_no_formats(self,
                         msg: str,
                         expected: bool = False,
                         video_id: str | None = None) -> None:
        ...

    @staticmethod
    def url_result(
        url: str,
        ie: InfoExtractor | None = None,
        video_id: str | None = None,
        video_title: str | None = None,
        *,
        url_transparent: bool = False,
        **kwargs: object,
    ) -> dict[str, object]:
        ...

    @classmethod
    def playlist_from_matches(
        cls,
        matches: object,
        playlist_id: str | None = None,
        playlist_title: str | None = None,
        getter: Callable[..., object] = ...,
        ie: InfoExtractor | None = None,
        video_kwargs: Mapping[str, object] | None = None,
        **kwargs: object,
    ) -> dict[str, object]:
        ...

    @staticmethod
    def playlist_result(
        entries: Iterable[_InfoDict],
        playlist_id: str | None = ...,
        playlist_title: str | None = ...,
        playlist_description: str | None = ...,
        *,
        multi_video: bool = ...,
        **kwargs: object,
    ) -> _InfoDict:
        ...

    def http_scheme(self) -> str:
        ...

    @classmethod
    def get_testcases(cls, include_onlymatching: bool = False) -> Iterator[dict[str, object]]:
        ...

    @classmethod
    def get_webpage_testcases(cls) -> Iterator[dict[str, object]]:
        ...

    @property
    def age_limit(cls) -> int:
        ...

    @classmethod
    def is_single_video(cls, url: str) -> bool:
        ...

    @classmethod
    def is_suitable(cls, age_limit: int) -> bool:
        ...

    @classmethod
    def description(cls,
                    *,
                    markdown: bool = True,
                    search_examples: Sequence[str] | None = None) -> str:
        ...

    def extract_subtitles(self, *args: object, **kwargs: object) -> list[dict[str, object]]:
        ...

    def _configuration_arg(self,
                           key: str,
                           default: object = ...,
                           *,
                           ie_key: str | None = ...,
                           casesense: bool = ...) -> object:
        ...

    # These are dynamically created.
    def _download_xml_handle(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = ...,
        errnote: str | None = ...,
        transform_source: Callable[..., str] | None = ...,
        fatal: bool = ...,
        encoding: str | None = ...,
        data: object = ...,
        headers: Mapping[str, str] = ...,
        query: Mapping[str, str] = ...,
        expected_status: int | None = ...,
        impersonate: str | None = ...,
        require_impersonation: bool = ...,
    ) -> tuple[ET.ElementTree, Response]:
        ...

    def _download_xml(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = ...,
        errnote: str | None = ...,
        transform_source: Callable[..., str] | None = ...,
        fatal: bool = ...,
        encoding: str | None = ...,
        data: object = ...,
        headers: Mapping[str, str] = ...,
        query: Mapping[str, str] = ...,
        expected_status: int | None = ...,
        impersonate: str | None = ...,
        require_impersonation: bool = ...,
    ) -> ET.ElementTree:
        ...

    def _download_socket_json_handle(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = ...,
        errnote: str | None = ...,
        transform_source: Callable[..., str] | None = ...,
        fatal: bool = ...,
        encoding: str | None = ...,
        data: object = ...,
        headers: Mapping[str, str] = ...,
        query: Mapping[str, str] = ...,
        expected_status: int | None = ...,
        impersonate: str | None = ...,
        require_impersonation: bool = ...,
    ) -> tuple[dict[str, object], Response]:
        ...

    def _download_socket_json(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = ...,
        errnote: str | None = ...,
        transform_source: Callable[..., str] | None = ...,
        fatal: bool = ...,
        encoding: str | None = ...,
        data: object = ...,
        headers: Mapping[str, str] = ...,
        query: Mapping[str, str] = ...,
        expected_status: int | None = ...,
        impersonate: str | None = ...,
        require_impersonation: bool = ...,
    ) -> dict[str, object]:
        ...

    def _download_json_handle(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = ...,
        errnote: str | None = ...,
        transform_source: Callable[..., str] | None = ...,
        fatal: bool = ...,
        encoding: str | None = ...,
        data: object = ...,
        headers: Mapping[str, str] = ...,
        query: Mapping[str, str] = ...,
        expected_status: int | None = ...,
        impersonate: str | None = ...,
        require_impersonation: bool = ...,
    ) -> tuple[dict[str, object], Response]:
        ...

    def _download_json(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = ...,
        errnote: str | None = ...,
        transform_source: Callable[..., str] | None = ...,
        fatal: bool = ...,
        encoding: str | None = ...,
        data: object = ...,
        headers: Mapping[str, str] = ...,
        query: Mapping[str, str] = ...,
        expected_status: int | None = ...,
        impersonate: str | None = ...,
        require_impersonation: bool = ...,
    ) -> dict[str, object]:
        ...

    def _download_webpage(
        self,
        url_or_request: str | Request,
        video_id: str,
        note: str | None = ...,
        errnote: str | None = ...,
        transform_source: Callable[..., str] | None = ...,
        fatal: bool = ...,
        encoding: str | None = ...,
        data: object = ...,
        headers: Mapping[str, str] = ...,
        query: Mapping[str, str] = ...,
        expected_status: int | None = ...,
        impersonate: str | None = ...,
        require_impersonation: bool = ...,
    ) -> str:
        ...

    def _parse_xml(
        self,
        xml_string: str,
        video_id: str,
        transform_source: Callable[..., str] | None = ...,
        fatal: bool = ...,
        errnote: str | None = ...,
    ) -> ET.Element:
        ...

    def _parse_mpd_formats(self,
                           mpd_doc: ET.Element,
                           mpd_id: str | None = ...,
                           mpd_base_url: str = ...,
                           mpd_url: str | None = ...) -> list[object]:
        ...

    def _real_extract(self, url: str) -> _InfoDict:
        ...

    @staticmethod
    def _availability(
        is_private: bool | None = ...,
        needs_premium: bool | None = ...,
        needs_subscription: bool | None = ...,
        needs_auth: bool | None = ...,
        is_unlisted: bool | None = ...,
    ) -> Literal['needs_auth', 'premium_only', 'private', 'public', 'subscriber_only',
                 'unlisted'] | None:
        ...

    def _request_webpage(
        self,
        url_or_req: str | Request,
        video_id: str,
        note: str | None = ...,
        errnote: str | None = ...,
        fatal: bool = ...,
        data: object = ...,
        headers: Mapping[str, str] = ...,
        query: Mapping[str, str] = ...,
        expected_status: int | None = ...,
    ) -> Response | Literal[False]:
        ...

    @classmethod
    def _match_id(cls, url: str) -> str:
        ...

    def _search_regex(
        self,
        pattern: str | re.Pattern[str],
        string: str | None,
        name: str,
        default: _StrNoDefaultOrNone = ...,
        fatal: bool = True,
        flags: int = 0,
        group: tuple[int, ...] | list[int] | None = None,
    ) -> str:
        ...

    def _search_json(
        self,
        start_pattern: str | re.Pattern[str],
        string: str | None,
        name: str,
        video_id: str,
        *,
        end_pattern: str | re.Pattern[str] = '',
        contains_pattern: str | re.Pattern[str] = r'{(?s:.+)}',
        fatal: bool = True,
        default: _StrNoDefaultOrNone = ...,
        **kwargs: object,
    ) -> object | None:
        ...

    def _html_search_regex(
        self,
        pattern: str | re.Pattern[str],
        string: str,
        name: str,
        default: _StrNoDefaultOrNone = ...,
        fatal: bool = True,
        flags: int = 0,
        group: int | None = None,
    ) -> str | tuple[str, ...]:
        ...

    def _get_netrc_login_info(self,
                              netrc_machine: str | None = None) -> tuple[str | None, str | None]:
        ...

    def _get_login_info(self,
                        username_option: str = 'username',
                        password_option: str = 'password',
                        netrc_machine: str | None = None) -> tuple[str | None, str | None]:
        ...

    def _get_tfa_info(self, note: str = 'two-factor verification code') -> str:
        ...

    @staticmethod
    def _og_regexes(prop: str) -> list[str]:
        ...

    @staticmethod
    def _meta_regex(prop: str) -> str:
        ...

    def _og_search_property(self,
                            prop: str,
                            html: str,
                            name: str | None = None,
                            **kargs: object) -> str | None:
        ...

    def _og_search_thumbnail(self, html: str, **kargs: object) -> str | None:
        ...

    def _og_search_description(self, html: str, **kargs: object) -> str | None:
        ...

    def _og_search_title(self, html: str, *, fatal: bool = False, **kargs: object) -> str | None:
        ...

    def _og_search_video_url(self,
                             html: str,
                             name: str = 'video url',
                             secure: bool = True,
                             **kargs: object) -> str | None:
        ...

    def _og_search_url(self, html: str, **kargs: object) -> str | None:
        ...

    def _html_extract_title(self,
                            html: str,
                            name: str = 'title',
                            *,
                            fatal: bool = False,
                            **kwargs: object) -> str | None:
        ...

    def _html_search_meta(self,
                          name: str,
                          html: str,
                          display_name: str | None = None,
                          fatal: bool = False,
                          **kwargs: object) -> str | None:
        ...

    def _dc_search_uploader(self, html: str) -> str | None:
        ...

    @staticmethod
    def _rta_search(html: str) -> int:
        ...

    def _media_rating_search(self, html: str) -> int:
        ...

    def _family_friendly_search(self, html: str) -> int:
        ...

    def _twitter_search_player(self, html: str) -> str | None:
        ...

    def _yield_json_ld(self,
                       html: str,
                       video_id: str,
                       *,
                       fatal: bool = True,
                       default: type[NO_DEFAULT] | bool = ...) -> Iterator[dict[str, object]]:
        ...

    def _search_json_ld(
        self,
        html: str,
        video_id: str,
        expected_type: Iterable[str] | str | None = None,
        *,
        fatal: bool = True,
        default: type[NO_DEFAULT] | bool = ...,
    ) -> dict[str, object]:
        ...

    def _json_ld(self,
                 json_ld: object,
                 video_id: str,
                 fatal: bool = True,
                 expected_type: Iterable[str] | str | None = None) -> dict[str, object]:
        ...

    def _search_nextjs_data(self,
                            webpage: str,
                            video_id: str,
                            *,
                            fatal: bool = True,
                            default: type[NO_DEFAULT] | bool = ...,
                            **kw: object) -> object:
        ...

    def _search_nuxt_data(
            self,
            webpage: str,
            video_id: str,
            context_name: str = '__NUXT__',
            *,
            fatal: bool = True,
            traverse: tuple[str, int] = ('data', 0),
    ) -> object:
        ...

    @staticmethod
    def _hidden_inputs(html: str) -> dict[str, object]:
        ...

    def _form_hidden_inputs(self, form_id: str, html: str) -> dict[str, object]:
        ...

    def _check_formats(self, formats: list[dict[str, object]], video_id: str) -> None:
        ...

    @staticmethod
    def _remove_duplicate_formats(formats: list[dict[str, object]]) -> None:
        ...

    def _is_valid_url(self,
                      url: str,
                      video_id: str,
                      item: str = 'video',
                      headers: Mapping[str, object] = ...) -> bool:
        ...

    def _proto_relative_url(self, url: str, scheme: str | None = None) -> str:
        ...

    def _sleep(self, timeout: float, video_id: str, msg_template: str | None = None) -> None:
        ...

    def _extract_f4m_formats(
        self,
        manifest_url: str,
        video_id: str,
        preference: object | None = None,
        quality: object | None = None,
        f4m_id: str | None = None,
        transform_source: Callable[..., str] = ...,
        fatal: bool = True,
        m3u8_id: str | None = None,
        data: str | None = None,
        headers: Mapping[str, object] = ...,
        query: Mapping[str, object] = ...,
    ) -> list[dict[str, object]]:
        ...

    def _parse_f4m_formats(
        self,
        manifest: str,
        manifest_url: str,
        video_id: str,
        preference: object | None = None,
        quality: object | None = None,
        f4m_id: str | None = None,
        transform_source: Callable[..., str] = ...,
        fatal: bool = True,
        m3u8_id: str | None = None,
    ) -> list[dict[str, object]]:
        ...

    def _m3u8_meta_format(
        self,
        m3u8_url: str,
        ext: str | None = None,
        preference: object | None = None,
        quality: object | None = None,
        m3u8_id: str | None = None,
    ) -> dict[str, object]:
        ...

    def _report_ignoring_subs(self, name: str) -> None:
        ...

    def _extract_m3u8_formats(self, *args: object, **kwargs: object) -> list[dict[str, object]]:
        ...

    def _extract_m3u8_formats_and_subtitles(
        self,
        m3u8_url: str,
        video_id: str,
        ext: str | None = None,
        entry_protocol: str = 'm3u8_native',
        preference: object | None = None,
        quality: object | None = None,
        m3u8_id: str | None = None,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        live: bool = False,
        data: object | None = None,
        headers: Mapping[str, object] = ...,
        query: Mapping[str, object] = ...,
    ) -> tuple[list[dict[str, object]], list[dict[str, object]]]:
        ...

    def _parse_m3u8_formats_and_subtitles(
        self,
        m3u8_doc: str,
        m3u8_url: str | None = None,
        ext: str | None = None,
        entry_protocol: str = 'm3u8_native',
        preference: object | None = None,
        quality: object | None = None,
        m3u8_id: str | None = None,
        live: bool = False,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        data: object | None = None,
        headers: Mapping[str, object] = ...,
        query: Mapping[str, object] = ...,
        video_id: str | None = None,
    ) -> tuple[list[dict[str, object]], list[dict[str, object]]]:
        ...

    def _extract_m3u8_vod_duration(
        self,
        m3u8_vod_url: str,
        video_id: str,
        note: str | None = None,
        errnote: str | None = None,
        data: object | None = None,
        headers: Mapping[str, object] = ...,
        query: Mapping[str, object] = ...,
    ) -> int | None:
        ...

    def _parse_m3u8_vod_duration(self, m3u8_vod: str, video_id: str) -> int:
        ...

    def _extract_mpd_vod_duration(
        self,
        mpd_url: str,
        video_id: str,
        note: str | None = None,
        errnote: str | None = None,
        data: object | None = None,
        headers: Mapping[str, object] = ...,
        query: Mapping[str, object] = ...,
    ) -> int | None:
        ...

    @staticmethod
    def _xpath_ns(path: str, namespace: str | None = None) -> str:
        ...

    def _extract_smil_formats_and_subtitles(
        self,
        smil_url: str,
        video_id: str,
        fatal: bool = True,
        f4m_params: Mapping[str, object] | None = None,
        transform_source: Callable[..., str] | None = None,
    ) -> tuple[list[dict[str, object]], list[dict[str, object]]]:
        ...

    def _extract_smil_formats(self, *args: object, **kwargs: object) -> list[dict[str, object]]:
        ...

    def _extract_smil_info(self,
                           smil_url: str,
                           video_id: str,
                           fatal: bool = True,
                           f4m_params: Mapping[str, object] | None = None) -> dict[str, object]:
        ...

    def _download_smil(self,
                       smil_url: str,
                       video_id: str,
                       fatal: bool = True,
                       transform_source: Callable[..., str] | None = None) -> ET.Element:
        ...

    def _parse_smil(self,
                    smil: ET.Element,
                    smil_url: str,
                    video_id: str,
                    f4m_params: Mapping[str, object] | None = None) -> dict[str, object]:
        ...

    def _parse_smil_namespace(self, smil: str) -> str | None:
        ...

    def _parse_smil_formats(self, *args: object, **kwargs: object) -> list[dict[str, object]]:
        ...

    def _parse_smil_formats_and_subtitles(
        self,
        smil: ET.Element,
        smil_url: str,
        video_id: str,
        namespace: str | None = None,
        f4m_params: Mapping[str, object] | None = None,
        transform_rtmp_url: Callable[[str, str], tuple[str, str]] | None = None,
    ) -> tuple[list[dict[str, object]], list[dict[str, object]]]:
        ...

    def _parse_smil_subtitles(self,
                              smil: ET.Element,
                              namespace: str | None = None,
                              subtitles_lang: str = 'en') -> list[dict[str, object]]:
        ...

    def _extract_xspf_playlist(self,
                               xspf_url: str,
                               playlist_id: str,
                               fatal: bool = True) -> list[dict[str, object]]:
        ...

    def _parse_xspf(self,
                    xspf_doc: ET.Element,
                    playlist_id: str,
                    xspf_url: str | None = None,
                    xspf_base_url: str | None = None) -> list[dict[str, object]]:
        ...

    def _extract_mpd_formats(self, *args: object, **kwargs: object) -> list[dict[str, object]]:
        ...

    def _extract_mpd_formats_and_subtitles(
            self, *args: object,
            **kwargs: object) -> tuple[list[dict[str, object]], list[dict[str, object]]]:
        ...

    def _extract_mpd_periods(
        self,
        mpd_url: str,
        video_id: str,
        mpd_id: str | None = None,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        data: object | None = None,
        headers: Mapping[str, object] = ...,
        query: Mapping[str, object] = ...,
    ) -> tuple[list[object], dict[str, object]]:
        ...

    def _parse_mpd_formats_and_subtitles(
            self, *args: object,
            **kwargs: object) -> tuple[list[dict[str, object]], list[dict[str, object]]]:
        ...

    def _merge_mpd_periods(
            self, periods: Iterable[Mapping[str,
                                            object]]) -> tuple[list[object], dict[str, object]]:
        ...

    def _parse_mpd_periods(self,
                           mpd_doc: ET.Element,
                           mpd_id: str | None = None,
                           mpd_base_url: str = '',
                           mpd_url: str | None = None) -> tuple[list[object], dict[str, object]]:
        ...

    def _extract_ism_formats(self, *args: object, **kwargs: object) -> list[dict[str, object]]:
        ...

    def _extract_ism_formats_and_subtitles(
        self,
        ism_url: str,
        video_id: str,
        ism_id: str | None = None,
        note: str | None = None,
        errnote: str | None = None,
        fatal: bool = True,
        data: object | None = None,
        headers: Mapping[str, object] = ...,
        query: Mapping[str, object] = ...,
    ) -> tuple[list[dict[str, object]], list[dict[str, object]]]:
        ...

    def _parse_ism_formats_and_subtitles(
            self,
            ism_doc: str,
            ism_url: str,
            ism_id: str | None = None) -> tuple[list[dict[str, object]], list[dict[str, object]]]:
        ...

    def _parse_html5_media_entries(
        self,
        base_url: str,
        webpage: str,
        video_id: str,
        m3u8_id: str | None = None,
        m3u8_entry_protocol: str = 'm3u8_native',
        mpd_id: str | None = None,
        preference: object | None = None,
        quality: object | None = None,
        _headers: Mapping[str, object] | None = None,
    ) -> list[dict[str, object]]:
        ...

    def _extract_akamai_formats(self, *args: object, **kwargs: object) -> list[dict[str, object]]:
        ...

    def _extract_akamai_formats_and_subtitles(
        self,
        manifest_url: str,
        video_id: str,
        hosts: Mapping[str,
                       object] = ...) -> tuple[list[dict[str, object]], list[dict[str, object]]]:
        ...

    def _extract_wowza_formats(self,
                               url: str,
                               video_id: str,
                               m3u8_entry_protocol: str = 'm3u8_native',
                               skip_protocols: Collection[str] = ...) -> list[dict[str, object]]:
        ...

    def _find_jwplayer_data(self,
                            webpage: str,
                            video_id: str | None = None,
                            transform_source: Callable[..., object] = ...) -> object | None:
        ...

    def _extract_jwplayer_data(self,
                               webpage: str,
                               video_id: str,
                               *args: object,
                               transform_source: Callable[..., object] = ...,
                               **kwargs: object) -> list[dict[str, object]]:
        ...

    def _parse_jwplayer_data(
        self,
        jwplayer_data: Mapping[str, object],
        video_id: str | None = None,
        require_title: bool = True,
        m3u8_id: str | None = None,
        mpd_id: str | None = None,
        rtmp_params: Mapping[str, object] | None = None,
        base_url: str | None = None,
    ) -> list[dict[str, object]]:
        ...

    def _parse_jwplayer_formats(
        self,
        jwplayer_sources_data: Iterable[Mapping[str, object]],
        video_id: str | None = None,
        m3u8_id: str | None = None,
        mpd_id: str | None = None,
        rtmp_params: Mapping[str, object] | None = None,
        base_url: str | None = None,
    ) -> list[dict[str, object]]:
        ...

    def _int(self, v: object, name: str, fatal: bool = False, **kwargs: object) -> int | None:
        ...

    def _float(self, v: object, name: str, fatal: bool = False, **kwargs: object) -> float | None:
        ...

    def _set_cookie(
        self,
        domain: str,
        name: str,
        value: str,
        expire_time: int | None = None,
        port: int | None = None,
        path: str = '/',
        secure: bool = False,
        discard: bool = False,
        rest: dict[str, object] = ...,
        **kwargs: object,
    ) -> None:
        ...

    def _live_title(self, name: _T) -> _T:
        ...

    def _get_cookies(self, url: str) -> LenientSimpleCookie:
        ...

    def _apply_first_set_cookie_header(self, url_handle: Response, cookie: str) -> None:
        ...

    @property
    def _RETURN_TYPE(cls) -> str:
        ...

    def _get_subtitles(self, *args: object, **kwargs: object) -> list[dict[str, object]]:
        ...

    def extract_comments(self, *args: object, **kwargs: object) -> list[dict[str, object]]:
        ...

    def _get_comments(self, *args: object, **kwargs: object) -> list[dict[str, object]]:
        ...

    @staticmethod
    def _merge_subtitle_items(
            subtitle_list1: Iterable[Mapping[str, object]],
            subtitle_list2: Iterable[Mapping[str, object]]) -> list[dict[str, object]]:
        ...

    @classmethod
    def _merge_subtitles(cls, *dicts: dict[str, object], target: object | None = None) -> object:
        ...

    def extract_automatic_captions(self, *args: object, **kwargs: object) -> dict[str, object]:
        ...

    @cached_property
    def _cookies_passed(self) -> bool:
        ...

    def _mark_watched(self, *args: object, **kwargs: object) -> object:
        ...

    @staticmethod
    def _generic_id(url: str) -> str:
        ...

    def _generic_title(self,
                       url: str = '',
                       webpage: str = '',
                       *,
                       default: str | None = None) -> str | None:
        ...

    def _extract_chapters_helper(
        self,
        chapter_list: Iterable[Mapping[str, object]],
        start_function: Callable[..., object],
        title_function: Callable[..., object],
        duration: float,
        strict: bool = True,
    ) -> list[dict[str, int]] | None:
        ...

    def _extract_chapters_from_description(self, description: str | None,
                                           duration: str | None) -> list[dict[str, int]] | None:
        ...

    def mark_watched(self, *args: object, **kwargs: object) -> None:
        ...

    def geo_verification_headers(self) -> dict[str, str]:
        ...

    def RetryManager(self, **kwargs: object) -> _RetryManager:
        ...

    @classmethod
    def extract_from_webpage(cls, ydl: YoutubeDL, url: str, webpage: str) -> Iterator[_InfoDict]:
        ...

    def _yes_playlist(
        self,
        playlist_id: str,
        video_id: str,
        smuggled_data: object | None = None,
        *,
        playlist_label: str = 'playlist',
        video_label: str = 'video',
    ) -> bool:
        ...

    def _error_or_warning(self,
                          err: str,
                          _count: int | None = None,
                          _retries: int = 0,
                          *,
                          fatal: bool = True) -> None:
        ...

    def _extract_generic_embeds(self,
                                url: str,
                                *args: object,
                                info_dict: _InfoDict = ...,
                                note: str = 'Extracting generic embeds',
                                **kwargs: object) -> list[dict[str, object]]:
        ...

    @classmethod
    def _extract_from_webpage(cls, url: str, webpage: str) -> Iterator[_InfoDict]:
        ...

    @classmethod
    def _extract_embed_urls(cls, url: str, webpage: str) -> Iterator[str]:
        ...

    @classmethod
    def _extract_url(cls, webpage: str) -> str | None:
        ...

    @classmethod
    def __init_subclass__(cls, *, plugin_name: str | None = None, **kwargs: object) -> None:
        ...

    class StopExtraction(Exception):
        ...

    class CommentsDisabled(Exception):
        ...


class SearchInfoExtractor(InfoExtractor):
    def _real_extract(self, query: str) -> _InfoDict:
        ...

    def _get_n_results(self, query: str, n: int) -> list[_InfoDict]:
        ...

    def _search_results(self, query: str) -> list[_InfoDict]:
        ...


class UnsupportedURLIE(InfoExtractor):
    ...
